# $Id: 00_initialize.t,v 1.1.1.1 2005/11/13 06:16:19 bryce Exp $

use strict;
use Test::More tests => 1;

# FOREACH perl_module_name...   # <-- TODO

#BEGIN { use_ok('[% perl_module_name %]'); }

#diag( "Testing [% perl_module_name %] $[% perl_module_name %]::VERSION" );

ok(1==1);
